@@ -1,5 +1,5 @@
 # ansisql.py
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -99,6 +99,9 @@ class ANSICompiler(sql.Compiled):
         # which will be passed to a ResultProxy and used for resultset-level value conversion
         self.typemap = {}
         
+        # a dictionary of select columns mapped to their name or key
+        self.columns = {}
+        
         # True if this compiled represents an INSERT
         self.isinsert = False
         
@@ -207,11 +210,12 @@ class ANSICompiler(sql.Compiled):
             # if we are within a visit to a Select, set up the "typemap"
             # for this column which is used to translate result set values
             self.typemap.setdefault(column.name.lower(), column.type)
+            self.columns.setdefault(column.key, column)
         if column.table is None or not column.table.named_with_column():
             self.strings[column] = self.preparer.format_column(column)
         else:
             if column.table.oid_column is column:
-                n = self.dialect.oid_column_name()
+                n = self.dialect.oid_column_name(column)
                 if n is not None:
                     self.strings[column] = "%s.%s" % (self.preparer.format_table(column.table, use_schema=False), n)
                 elif len(column.table.primary_key) != 0:
@@ -321,7 +325,7 @@ class ANSICompiler(sql.Compiled):
         # redefine the generated name of the bind param in the case
         # that we have multiple conflicting bind parameters.
         while self.binds.setdefault(key, bindparam) is not bindparam:
-            # insure the name doesn't expand the length of the string
+            # ensure the name doesn't expand the length of the string
             # in case we're at the edge of max identifier length
             tag = "_%d" % count
             key = bindparam.key[0 : len(bindparam.key) - len(tag)] + tag
@@ -673,7 +677,7 @@ class ANSISchemaGenerator(ANSISchemaBase):
         raise NotImplementedError()
     
     def visit_metadata(self, metadata):
-        collection = [t for t in metadata.table_iterator(reverse=False, tables=self.tables) if (not self.checkfirst or not self.dialect.has_table(self.connection, t.name))]
+        collection = [t for t in metadata.table_iterator(reverse=False, tables=self.tables) if (not self.checkfirst or not self.dialect.has_table(self.connection, t.name, schema=t.schema))]
         for table in collection:
             table.accept_schema_visitor(self, traverse=False)
         if self.supports_alter():
@@ -701,8 +705,12 @@ class ANSISchemaGenerator(ANSISchemaBase):
                 first_pk = True
             for constraint in column.constraints:
                 constraint.accept_schema_visitor(self, traverse=False)
-                
-        for constraint in table.constraints:
+
+        # On some DB order is significant: visit PK first, then the
+        # other constraints (engine.ReflectionTest.testbasic failed on FB2)
+        if len(table.primary_key):
+            table.primary_key.accept_schema_visitor(self, traverse=False)
+        for constraint in [c for c in table.constraints if c is not table.primary_key]:
             constraint.accept_schema_visitor(self, traverse=False)
 
         self.append("\n)%s\n\n" % self.post_create_table(table))
@@ -734,13 +742,18 @@ class ANSISchemaGenerator(ANSISchemaBase):
         if constraint.name is not None:
             self.append("CONSTRAINT %s " % constraint.name)
         self.append(" CHECK (%s)" % constraint.sqltext)
+
+    def visit_column_check_constraint(self, constraint):
+        self.append(" ")
+        self.append(" CHECK (%s)" % constraint.sqltext)
         
     def visit_primary_key_constraint(self, constraint):
         if len(constraint) == 0:
             return
-        self.append(", \n\tPRIMARY KEY ")
+        self.append(", \n\t")
         if constraint.name is not None:
-            self.append("%s " % constraint.name)
+            self.append("CONSTRAINT %s " % constraint.name)
+        self.append("PRIMARY KEY ")
         self.append("(%s)" % (string.join([self.preparer.format_column(c) for c in constraint],', ')))
     
     def supports_alter(self):
@@ -799,7 +812,7 @@ class ANSISchemaDropper(ANSISchemaBase):
         self.dialect = self.engine.dialect
 
     def visit_metadata(self, metadata):
-        collection = [t for t in metadata.table_iterator(reverse=True, tables=self.tables) if (not self.checkfirst or  self.dialect.has_table(self.connection, t.name))]
+        collection = [t for t in metadata.table_iterator(reverse=True, tables=self.tables) if (not self.checkfirst or  self.dialect.has_table(self.connection, t.name, schema=t.schema))]
         if self.supports_alter():
             for alterable in self.find_alterables(collection):
                 self.drop_foreignkey(alterable)
@@ -924,9 +937,7 @@ class ANSIIdentifierPreparer(object):
     
     def format_column(self, column, use_table=False):
         """Prepare a quoted column name """
-        # TODO: isinstance alert !  get ColumnClause and Column to better
-        # differentiate themselves
-        if isinstance(column, schema.SchemaItem):
+        if not getattr(column, 'is_literal', False):
             if use_table:
                 return self.format_table(column.table, use_schema=False) + "." + self.__generic_obj_format(column, column.name)
             else:
@@ -934,7 +945,7 @@ class ANSIIdentifierPreparer(object):
         else:
             # literal textual elements get stuck into ColumnClause alot, which shouldnt get quoted
             if use_table:
-                return column.table.name + "." + column.name
+                return self.format_table(column.table, use_schema=False) + "." + column.name
             else:
                 return column.name
             
