@@ -1,5 +1,5 @@
 # oracle.py
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -7,23 +7,24 @@
 
 import sys, StringIO, string, re
 
-import sqlalchemy.util as util
-import sqlalchemy.sql as sql
-import sqlalchemy.engine as engine
+from sqlalchemy import util, sql, engine, schema, ansisql, exceptions, logging
 import sqlalchemy.engine.default as default
-import sqlalchemy.schema as schema
-import sqlalchemy.ansisql as ansisql
 import sqlalchemy.types as sqltypes
-import sqlalchemy.exceptions as exceptions
 
 try:
     import cx_Oracle
 except:
     cx_Oracle = None
+
+ORACLE_BINARY_TYPES = [getattr(cx_Oracle, k) for k in ["BFILE", "CLOB", "NCLOB", "BLOB", "LONG_BINARY", "LONG_STRING"] if hasattr(cx_Oracle, k)]
+
         
 class OracleNumeric(sqltypes.Numeric):
     def get_col_spec(self):
-        return "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length}
+        if self.precision is None:
+            return "NUMERIC"
+        else:
+            return "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length}
 class OracleInteger(sqltypes.Integer):
     def get_col_spec(self):
         return "INTEGER"
@@ -58,8 +59,22 @@ class OracleChar(sqltypes.CHAR):
     def get_col_spec(self):
         return "CHAR(%(length)s)" % {'length' : self.length}
 class OracleBinary(sqltypes.Binary):
+    def get_dbapi_type(self, dbapi):
+        return dbapi.BINARY
     def get_col_spec(self):
         return "BLOB"
+    def convert_bind_param(self, value, dialect):
+        if value is None:
+            return None
+        else:
+            # this is RAWTOHEX
+            return ''.join(["%.2X" % ord(c) for c in value])
+    def convert_result_value(self, value, dialect):
+        if value is None:
+            return None
+        else:
+            return value.read()
+
 class OracleBoolean(sqltypes.Boolean):
     def get_col_spec(self):
         return "SMALLINT"
@@ -106,25 +121,6 @@ ischema_names = {
     'DOUBLE PRECISION' : OracleNumeric,
 }
 
-constraintSQL = """SELECT
-  ac.constraint_name,
-  ac.constraint_type,
-  LOWER(loc.column_name) AS local_column,
-  LOWER(rem.table_name) AS remote_table,
-  LOWER(rem.column_name) AS remote_column,
-  LOWER(rem.owner) AS remote_owner
-FROM all_constraints ac,
-  all_cons_columns loc,
-  all_cons_columns rem
-WHERE ac.table_name = :table_name
-AND ac.constraint_type IN ('R','P')
-AND ac.owner = :owner
-AND ac.owner = loc.owner
-AND ac.constraint_name = loc.constraint_name
-AND ac.r_owner = rem.owner(+)
-AND ac.r_constraint_name = rem.constraint_name(+)
--- order multiple primary keys correctly
-ORDER BY ac.constraint_name, loc.position, rem.position"""
 
 
 def descriptor():
@@ -181,8 +177,11 @@ class OracleDialect(ansisql.ANSIDialect):
     def type_descriptor(self, typeobj):
         return sqltypes.adapt_type(typeobj, colspecs)
 
-    def oid_column_name(self):
-        return "rowid"
+    def oid_column_name(self, column):
+        if not isinstance(column.table, sql.TableClause) and not isinstance(column.table, sql.Select):
+            return None
+        else:
+            return "rowid"
 
     def create_execution_context(self):
         return OracleExecutionContext(self)
@@ -197,13 +196,72 @@ class OracleDialect(ansisql.ANSIDialect):
         return OracleDefaultRunner(engine, proxy)
 
 
-    def has_table(self, connection, table_name):
+    def has_table(self, connection, table_name, schema=None):
         cursor = connection.execute("""select table_name from all_tables where table_name=:name""", {'name':table_name.upper()})
         return bool( cursor.fetchone() is not None )
 
     def has_sequence(self, connection, sequence_name):
         cursor = connection.execute("""select sequence_name from all_sequences where sequence_name=:name""", {'name':sequence_name.upper()})
         return bool( cursor.fetchone() is not None )
+
+    def _locate_owner_row(self, owner, name, rows, raiseerr=False):
+        """return the row in the given list of rows which references the given table name and owner name."""
+        if not rows:
+            if raiseerr:
+                raise exceptions.NoSuchTableError(name)
+            else:
+                return None
+        else:
+            if owner is not None:
+                for row in rows:
+                    if owner.upper() in row[0]:
+                        return row
+                else:
+                    if raiseerr:
+                        raise exceptions.AssertionError("Specified owner %s does not own table %s" % (owner, name))
+                    else:
+                        return None
+            else:
+                if len(rows)==1:
+                    return rows[0]
+                else:
+                    if raiseerr:
+                        raise exceptions.AssertionError("There are multiple tables with name '%s' visible to the schema, you must specifiy owner" % name)
+                    else:
+                        return None
+    def _resolve_table_owner(self, connection, name, table, dblink=''):
+        """locate the given table in the ALL_TAB_COLUMNS view, including searching for equivalent synonyms and dblinks"""
+        c = connection.execute ("select distinct OWNER from ALL_TAB_COLUMNS%(dblink)s where TABLE_NAME = :table_name" % {'dblink':dblink}, {'table_name':name})
+        rows = c.fetchall()
+        try:
+            row = self._locate_owner_row(table.owner, name, rows, raiseerr=True)
+            return name, row['OWNER'], ''
+        except exceptions.SQLAlchemyError:
+            # locate synonyms
+            c = connection.execute ("""select OWNER, TABLE_OWNER, TABLE_NAME, DB_LINK
+                                       from   ALL_SYNONYMS%(dblink)s 
+                                       where  SYNONYM_NAME = :synonym_name
+                                       and (DB_LINK IS NOT NULL 
+                                               or ((TABLE_NAME, TABLE_OWNER) in 
+                                                    (select TABLE_NAME, OWNER from ALL_TAB_COLUMNS%(dblink)s)))""" % {'dblink':dblink},
+                                    {'synonym_name':name})
+            rows = c.fetchall()
+            row = self._locate_owner_row(table.owner, name, rows)
+            if row is None:
+                row = self._locate_owner_row("PUBLIC", name, rows)
+
+            if row is not None:
+                owner, name, dblink = row['TABLE_OWNER'], row['TABLE_NAME'], row['DB_LINK']
+                if dblink:
+                    dblink = '@' + dblink
+                    if not owner:
+                        # re-resolve table owner using new dblink variable
+                        t1, owner, t2 = self._resolve_table_owner(connection, name, table, dblink=dblink)
+                else:
+                    dblink = ''
+                return name, owner, dblink
+            raise
+
         
     def reflecttable(self, connection, table):
         preparer = self.identifier_preparer
@@ -211,23 +269,12 @@ class OracleDialect(ansisql.ANSIDialect):
             name = table.name.upper()
         else:
             name = table.name
-        c = connection.execute ("select distinct OWNER from ALL_TAB_COLUMNS where TABLE_NAME = :table_name", {'table_name':name})
-        rows = c.fetchall()
-        if not rows :
-            raise exceptions.NoSuchTableError(table.name)
-        else:
-            if table.owner is not None:
-                if table.owner.upper() in [r[0] for r in rows]:
-                    owner = table.owner.upper()
-                else:
-                    raise exceptions.AssertionError("Specified owner %s does not own table %s"%(table.owner, table.name))
-            else:
-                if len(rows)==1:
-                    owner = rows[0][0]
-                else:
-                    raise exceptions.AssertionError("There are multiple tables with name %s in the schema, you must specifie owner"%table.name)
-
-        c = connection.execute ("select COLUMN_NAME, DATA_TYPE, DATA_LENGTH, DATA_PRECISION, DATA_SCALE, NULLABLE, DATA_DEFAULT from ALL_TAB_COLUMNS where TABLE_NAME = :table_name and OWNER = :owner", {'table_name':name, 'owner':owner})
+        
+        # search for table, including across synonyms and dblinks.
+        # locate the actual name of the table, the real owner, and any dblink clause needed.
+        actual_name, owner, dblink = self._resolve_table_owner(connection, name, table)
+        
+        c = connection.execute ("select COLUMN_NAME, DATA_TYPE, DATA_LENGTH, DATA_PRECISION, DATA_SCALE, NULLABLE, DATA_DEFAULT from ALL_TAB_COLUMNS%(dblink)s where TABLE_NAME = :table_name and OWNER = :owner" % {'dblink':dblink}, {'table_name':actual_name, 'owner':owner})
         
         while True:
             row = c.fetchone()
@@ -236,7 +283,7 @@ class OracleDialect(ansisql.ANSIDialect):
             found_table = True
 
             #print "ROW:" , row
-            (name, coltype, length, precision, scale, nullable, default) = (row[0], row[1], row[2], row[3], row[4], row[5]=='Y', row[6])
+            (colname, coltype, length, precision, scale, nullable, default) = (row[0], row[1], row[2], row[3], row[4], row[5]=='Y', row[6])
 
             # INTEGER if the scale is 0 and precision is null
             # NUMBER if the scale and precision are both null
@@ -257,20 +304,42 @@ class OracleDialect(ansisql.ANSIDialect):
                 try:
                     coltype = ischema_names[coltype]
                 except KeyError:
-                    raise exceptions.AssertionError("Cant get coltype for type '%s'" % coltype)
+                    raise exceptions.AssertionError("Cant get coltype for type '%s' on colname '%s'" % (coltype, colname))
                
             colargs = []
             if default is not None:
                 colargs.append(schema.PassiveDefault(sql.text(default)))
           
             # if name comes back as all upper, assume its case folded 
-            if (name.upper() == name): 
-                name = name.lower()
+            if (colname.upper() == colname): 
+                colname = colname.lower()
             
-            table.append_column(schema.Column(name, coltype, nullable=nullable, *colargs))
+            table.append_column(schema.Column(colname, coltype, nullable=nullable, *colargs))
 
-       
-        c = connection.execute(constraintSQL, {'table_name' : table.name.upper(), 'owner' : owner})
+        if not len(table.columns):
+           raise exceptions.AssertionError("Couldn't find any column information for table %s" % actual_name)
+           
+        c = connection.execute("""SELECT
+             ac.constraint_name,
+             ac.constraint_type,
+             LOWER(loc.column_name) AS local_column,
+             LOWER(rem.table_name) AS remote_table,
+             LOWER(rem.column_name) AS remote_column,
+             LOWER(rem.owner) AS remote_owner
+           FROM all_constraints%(dblink)s ac,
+             all_cons_columns%(dblink)s loc,
+             all_cons_columns%(dblink)s rem
+           WHERE ac.table_name = :table_name
+           AND ac.constraint_type IN ('R','P')
+           AND ac.owner = :owner
+           AND ac.owner = loc.owner
+           AND ac.constraint_name = loc.constraint_name
+           AND ac.r_owner = rem.owner(+)
+           AND ac.r_constraint_name = rem.constraint_name(+)
+           -- order multiple primary keys correctly
+           ORDER BY ac.constraint_name, loc.position, rem.position"""
+         % {'dblink':dblink}, {'table_name' : actual_name, 'owner' : owner})
+         
         fks = {}
         while True:
             row = c.fetchone()
@@ -286,6 +355,9 @@ class OracleDialect(ansisql.ANSIDialect):
                 except KeyError:
                    fk = ([], [])
                    fks[cons_name] = fk
+                if remote_table is None:
+                    # ticket 363
+                    self.logger.warn("Got 'None' querying 'table_name' from all_cons_columns%(dblink)s - does the user have proper rights to the table?" % {'dblink':dblink})
                 refspec = ".".join([remote_table, remote_column])
                 schema.Table(remote_table, table.metadata, autoload=True, autoload_with=connection, owner=remote_owner)
                 if local_column not in fk[0]:
@@ -304,6 +376,18 @@ class OracleDialect(ansisql.ANSIDialect):
         if context is not None:
             context._rowcount = rowcount
 
+    def create_result_proxy_args(self, connection, cursor):
+        args = super(OracleDialect, self).create_result_proxy_args(connection, cursor)
+        if cursor and cursor.description:
+            for column in cursor.description:
+                type_code = column[1]
+                if type_code in ORACLE_BINARY_TYPES:
+                    args['should_prefetch'] = True
+                    break
+        return args
+
+OracleDialect.logger = logging.class_logger(OracleDialect)
+
 class OracleCompiler(ansisql.ANSICompiler):
     """oracle compiler modifies the lexical structure of Select statements to work under 
     non-ANSI configured Oracle databases, if the use_ansi flag is False."""
@@ -335,6 +419,11 @@ class OracleCompiler(ansisql.ANSICompiler):
             self._outertable = None
 
         self.visit_compound(self.wheres[join])
+
+    def visit_insert_sequence(self, column, sequence, parameters):
+        """this is the 'sequence' equivalent to ANSICompiler's 'visit_insert_column_default' which ensures
+        that the column is present in the generated column list"""
+        parameters.setdefault(column.key, None)
        
     def visit_alias(self, alias):
 	"""oracle doesnt like 'FROM table AS alias'.  is the AS standard SQL??"""
@@ -375,7 +464,7 @@ class OracleCompiler(ansisql.ANSICompiler):
                 orderby = self.strings[orderby]
             class SelectVisitor(sql.ClauseVisitor):
                 def visit_select(self, select):
-                    select.append_column(sql.column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
+                    select.append_column(sql.literal_column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
             select.accept_visitor(SelectVisitor())
             limitselect = sql.select([c for c in select.c if c.key!='ora_rn'])
             if select.offset is not None:
@@ -410,7 +499,7 @@ class OracleCompiler(ansisql.ANSICompiler):
                 orderby = select.oid_column
                 orderby.accept_visitor(self)
                 orderby = self.strings[orderby]
-            select.append_column(sql.column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
+            select.append_column(sql.literal_column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
             limitselect = sql.select([c for c in select.c if c.key!='ora_rn'])
             if select.offset is not None:
                 limitselect.append_whereclause("ora_rn>%d" % select.offset)
@@ -450,6 +539,7 @@ class OracleSchemaGenerator(ansisql.ANSISchemaGenerator):
             self.append("CREATE SEQUENCE %s" % self.preparer.format_sequence(sequence))
             self.execute()
 
+     
 class OracleSchemaDropper(ansisql.ANSISchemaDropper):
     def visit_sequence(self, sequence):
         if self.engine.dialect.has_sequence(self.connection, sequence.name):
