@@ -1,5 +1,5 @@
 # engine/default.py
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -24,14 +24,13 @@ class PoolConnectionProvider(base.ConnectionProvider):
         
 class DefaultDialect(base.Dialect):
     """default implementation of Dialect"""
-    def __init__(self, convert_unicode=False, encoding='utf-8', **kwargs):
+    def __init__(self, convert_unicode=False, encoding='utf-8', default_paramstyle='named', **kwargs):
         self.convert_unicode = convert_unicode
         self.supports_autoclose_results = True
         self.encoding = encoding
         self.positional = False
-        self.paramstyle = 'named'
         self._ischema = None
-        self._figure_paramstyle()
+        self._figure_paramstyle(default=default_paramstyle)
     def create_execution_context(self):
         return DefaultExecutionContext(self)
     def type_descriptor(self, typeobj):
@@ -41,7 +40,7 @@ class DefaultDialect(base.Dialect):
         if type(typeobj) is type:
             typeobj = typeobj()
         return typeobj
-    def oid_column_name(self):
+    def oid_column_name(self, column):
         return None
     def supports_sane_rowcount(self):
         return True
@@ -64,6 +63,10 @@ class DefaultDialect(base.Dialect):
         cursor.execute(statement, parameters)
     def defaultrunner(self, engine, proxy):
         return base.DefaultRunner(engine, proxy)
+    def create_cursor(self, connection):
+        return connection.cursor()
+    def create_result_proxy_args(self, connection, cursor):
+        return dict(should_prefetch=False)
         
     def _set_paramstyle(self, style):
         self._paramstyle = style
@@ -88,14 +91,14 @@ class DefaultDialect(base.Dialect):
                     parameters = parameters.get_raw_dict()
         return parameters
 
-    def _figure_paramstyle(self, paramstyle=None):
+    def _figure_paramstyle(self, paramstyle=None, default='named'):
         db = self.dbapi()
         if paramstyle is not None:
             self._paramstyle = paramstyle
         elif db is not None:
             self._paramstyle = db.paramstyle
         else:
-            self._paramstyle = 'named'
+            self._paramstyle = default
 
         if self._paramstyle == 'named':
             self.positional=False
