@@ -1,12 +1,12 @@
 # attributes.py - manages object attributes
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 from sqlalchemy import util
 from sqlalchemy.orm import util as orm_util
-from sqlalchemy import logging
+from sqlalchemy import logging, exceptions
 import weakref
 
 class InstrumentedAttribute(object):
@@ -210,7 +210,7 @@ class InstrumentedAttribute(object):
                     orig = state.get('original', None)
                     if orig is not None:
                         orig.commit_attribute(self, obj)
-                    return obj.__dict__[self.key]
+                    return value
                 else:
                     # note that we arent raising AttributeErrors, just returning None.
                     # this might be a good thing to be changeable by options.
@@ -331,6 +331,8 @@ class InstrumentedList(object):
         elif hasattr(self.data, 'add'):
             self._data_appender = self.data.add
             self._clear_data = self._clear_set
+        else:
+            raise exceptions.ArgumentError("Collection type " + repr(type(self.data)) + " has no append() or add() method")
         if isinstance(self.data, dict):
             self._clear_data = self._clear_dict
             
@@ -758,16 +760,12 @@ class AttributeManager(object):
         #print self, "register attribute", key, "for class", class_
         if not hasattr(class_, '_state'):
             def _get_state(self):
-                try:
-                    return self.__sa_attr_state
-                except AttributeError:
-                    self.__sa_attr_state = {}
-                    return self.__sa_attr_state
+                if not hasattr(self, '_sa_attr_state'):
+                    self._sa_attr_state = {}
+                return self._sa_attr_state
             class_._state = property(_get_state)
         
         typecallable = kwargs.pop('typecallable', None)
-        if typecallable is None:
-            typecallable = getattr(class_, key, None)
         if isinstance(typecallable, InstrumentedAttribute):
             typecallable = None
         setattr(class_, key, self.create_prop(class_, key, uselist, callable_, typecallable=typecallable, **kwargs))
