@@ -1,5 +1,5 @@
 # orm/dependency.py
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -35,12 +35,16 @@ class DependencyProcessor(object):
         self.direction = prop.direction
         self.is_backref = prop.is_backref
         self.post_update = prop.post_update
-        self.foreignkey = prop.foreignkey
+        self.foreign_keys = prop.foreign_keys
         self.passive_deletes = prop.passive_deletes
         self.key = prop.key
 
         self._compile_synchronizers()
 
+    def _get_instrumented_attribute(self):
+        """return the InstrumentedAttribute handled by this DependencyProecssor"""
+        return getattr(self.parent.class_, self.key)
+        
     def register_dependencies(self, uowcommit):
         """tells a UOWTransaction what mappers are dependent on which, with regards
         to the two or three mappers handled by this PropertyLoader.
@@ -70,11 +74,11 @@ class DependencyProcessor(object):
         raise NotImplementedError()
 
     def preprocess_dependencies(self, task, deplist, uowcommit, delete = False):
-        """used before the flushes' topological sort to traverse through related objects and insure every 
+        """used before the flushes' topological sort to traverse through related objects and ensure every 
         instance which will require save/update/delete is properly added to the UOWTransaction."""
         raise NotImplementedError()
 
-    def _synchronize(self, obj, child, associationrow, clearkeys):
+    def _synchronize(self, obj, child, associationrow, clearkeys, uowcommit):
         """called during a flush to synchronize primary key identifier values between a parent/child object, as well as 
         to an associationrow in the case of many-to-many."""
         raise NotImplementedError()
@@ -88,10 +92,10 @@ class DependencyProcessor(object):
         objects are processed."""
         self.syncrules = sync.ClauseSynchronizer(self.parent, self.mapper, self.direction)
         if self.direction == sync.MANYTOMANY:
-            self.syncrules.compile(self.prop.primaryjoin, issecondary=False)
-            self.syncrules.compile(self.prop.secondaryjoin, issecondary=True)
+            self.syncrules.compile(self.prop.primaryjoin, issecondary=False, foreign_keys=self.foreign_keys)
+            self.syncrules.compile(self.prop.secondaryjoin, issecondary=True, foreign_keys=self.foreign_keys)
         else:
-            self.syncrules.compile(self.prop.primaryjoin, foreignkey=self.foreignkey)
+            self.syncrules.compile(self.prop.primaryjoin, foreign_keys=self.foreign_keys)
         
     def get_object_dependencies(self, obj, uowcommit, passive = True):
         """returns the list of objects that are dependent on the given object, as according to the relationship
@@ -108,7 +112,7 @@ class DependencyProcessor(object):
         given related object list contains INSERTs or DELETEs."""
         if obj is not None and self.post_update:
             for x in related:
-                if x is not None and (uowcommit.is_deleted(x) or not hasattr(x, '_instance_key')):
+                if x is not None:
                     uowcommit.register_object(obj, postupdate=True, post_update_cols=self.syncrules.dest_columns())
                     break
 
@@ -134,22 +138,22 @@ class OneToManyDP(DependencyProcessor):
                     if childlist is not None:
                         for child in childlist.deleted_items():
                             if child is not None and childlist.hasparent(child) is False:
-                                self._synchronize(obj, child, None, True)
+                                self._synchronize(obj, child, None, True, uowcommit)
                                 self._conditional_post_update(child, uowcommit, [obj])
                         for child in childlist.unchanged_items():
                             if child is not None:
-                                self._synchronize(obj, child, None, True)
+                                self._synchronize(obj, child, None, True, uowcommit)
                                 self._conditional_post_update(child, uowcommit, [obj])
         else:
             for obj in deplist:
                 childlist = self.get_object_dependencies(obj, uowcommit, passive=True)
                 if childlist is not None:
                     for child in childlist.added_items():
-                        self._synchronize(obj, child, None, False)
+                        self._synchronize(obj, child, None, False, uowcommit)
                         self._conditional_post_update(child, uowcommit, [obj])
                     for child in childlist.deleted_items():
-                        if not self.cascade.delete_orphan:
-                            self._synchronize(obj, child, None, True)
+                        if not self.cascade.delete_orphan and not self._get_instrumented_attribute().hasparent(child):
+                            self._synchronize(obj, child, None, True, uowcommit)
 
     def preprocess_dependencies(self, task, deplist, uowcommit, delete = False):
         #print self.mapper.mapped_table.name + " " + self.key + " " + repr(len(deplist)) + " preprocess_dep isdelete " + repr(delete) + " direction " + repr(self.direction)
@@ -198,10 +202,10 @@ class OneToManyDP(DependencyProcessor):
                             for c in self.mapper.cascade_iterator('delete', child):
                                 uowcommit.register_object(c, isdelete=True)
             
-    def _synchronize(self, obj, child, associationrow, clearkeys):
+    def _synchronize(self, obj, child, associationrow, clearkeys, uowcommit):
         source = obj
         dest = child
-        if dest is None:
+        if dest is None or (not self.post_update and uowcommit.is_deleted(dest)):
             return
         self.syncrules.execute(source, dest, obj, child, clearkeys)
     
@@ -223,7 +227,7 @@ class ManyToOneDP(DependencyProcessor):
                 # post_update means we have to update our row to not reference the child object
                 # before we can DELETE the row
                 for obj in deplist:
-                    self._synchronize(obj, None, None, True)
+                    self._synchronize(obj, None, None, True, uowcommit)
                     childlist = self.get_object_dependencies(obj, uowcommit, passive=self.passive_deletes)
                     if childlist is not None:
                         self._conditional_post_update(obj, uowcommit, childlist.deleted_items() + childlist.unchanged_items() + childlist.added_items())
@@ -232,9 +236,8 @@ class ManyToOneDP(DependencyProcessor):
                 childlist = self.get_object_dependencies(obj, uowcommit, passive=True)
                 if childlist is not None:
                     for child in childlist.added_items():
-                        self._synchronize(obj, child, None, False)
+                        self._synchronize(obj, child, None, False, uowcommit)
                     self._conditional_post_update(obj, uowcommit, childlist.deleted_items() + childlist.unchanged_items() + childlist.added_items())
-                        
     def preprocess_dependencies(self, task, deplist, uowcommit, delete = False):
         #print self.mapper.mapped_table.name + " " + self.key + " " + repr(len(deplist)) + " PRE process_dep isdelete " + repr(delete) + " direction " + repr(self.direction)
         if self.post_update:
@@ -261,10 +264,10 @@ class ManyToOneDP(DependencyProcessor):
                                 for c in self.mapper.cascade_iterator('delete', child):
                                     uowcommit.register_object(c, isdelete=True)
                         
-    def _synchronize(self, obj, child, associationrow, clearkeys):
+    def _synchronize(self, obj, child, associationrow, clearkeys, uowcommit):
         source = child
         dest = obj
-        if dest is None:
+        if dest is None or (not self.post_update and uowcommit.is_deleted(dest)):
             return
         self.syncrules.execute(source, dest, obj, child, clearkeys)
 
@@ -296,7 +299,7 @@ class ManyToManyDP(DependencyProcessor):
                 if childlist is not None:
                     for child in childlist.deleted_items() + childlist.unchanged_items():
                         associationrow = {}
-                        self._synchronize(obj, child, associationrow, False)
+                        self._synchronize(obj, child, associationrow, False, uowcommit)
                         secondary_delete.append(associationrow)
         else:
             for obj in deplist:
@@ -304,11 +307,11 @@ class ManyToManyDP(DependencyProcessor):
                 if childlist is None: continue
                 for child in childlist.added_items():
                     associationrow = {}
-                    self._synchronize(obj, child, associationrow, False)
+                    self._synchronize(obj, child, associationrow, False, uowcommit)
                     secondary_insert.append(associationrow)
                 for child in childlist.deleted_items():
                     associationrow = {}
-                    self._synchronize(obj, child, associationrow, False)
+                    self._synchronize(obj, child, associationrow, False, uowcommit)
                     secondary_delete.append(associationrow)
         if len(secondary_delete):
             secondary_delete.sort()
@@ -320,8 +323,17 @@ class ManyToManyDP(DependencyProcessor):
             connection.execute(statement, secondary_insert)
 
     def preprocess_dependencies(self, task, deplist, uowcommit, delete = False):
-        pass
-    def _synchronize(self, obj, child, associationrow, clearkeys):
+        #print self.mapper.mapped_table.name + " " + self.key + " " + repr(len(deplist)) + " preprocess_dep isdelete " + repr(delete) + " direction " + repr(self.direction)
+        if not delete:
+            for obj in deplist:
+                childlist = self.get_object_dependencies(obj, uowcommit, passive=True)
+                if childlist is not None:
+                    for child in childlist.deleted_items():
+                        if self.cascade.delete_orphan and childlist.hasparent(child) is False:
+                            uowcommit.register_object(child, isdelete=True)
+                            for c in self.mapper.cascade_iterator('delete', child):
+                                uowcommit.register_object(c, isdelete=True)
+    def _synchronize(self, obj, child, associationrow, clearkeys, uowcommit):
         dest = associationrow
         source = None
         if dest is None:
