@@ -1,5 +1,5 @@
 # objectstore.py
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -69,6 +69,15 @@ class SessionTransaction(object):
             if t[2]:
                 t[0].close()
         self.session.transaction = None
+    def __enter__(self):
+        return self
+    def __exit__(self, type, value, traceback):
+        if self.session.transaction is None:
+            return
+        if type is None:
+            self.commit()
+        else:
+            self.rollback()
 
 class Session(object):
     """encapsulates a set of objects being operated upon within an object-relational operation.
@@ -323,30 +332,30 @@ class Session(object):
         for c in [object] + list(_object_mapper(object).cascade_iterator('delete', object)):
             self.uow.register_deleted(c)
 
-    def merge(self, object, entity_name=None):
-        """merge the object into a newly loaded or existing instance from this Session.
+    def merge(self, object, entity_name=None, _recursive=None):
+        """copy the state of the given object onto the persistent object with the same identifier. 
         
-        note: this method is currently not completely implemented."""
-        instance = None
-        for obj in [object] + list(_object_mapper(object).cascade_iterator('merge', object)):
-            key = getattr(obj, '_instance_key', None)
-            if key is None:
-                mapper = _object_mapper(object)
-                ident = mapper.identity(object)
-                for k in ident:
-                    if k is None:
-                        raise exceptions.InvalidRequestError("Instance '%s' does not have a full set of identity values, and does not represent a saved entity in the database.  Use the add() method to add unsaved instances to this Session." % repr(obj))
-                key = mapper.identity_key(ident)
-            u = self.uow
-            if u.identity_map.has_key(key):
-                # TODO: copy the state of the given object into this one.  tricky !
-                inst = u.identity_map[key]
+        If there is no persistent instance currently associated with the session, it will be loaded. 
+        Return the persistent instance. If the given instance is unsaved, save a copy of and return it as 
+        a newly persistent instance. The given instance does not become associated with the session. 
+        This operation cascades to associated instances if the association is mapped with cascade="merge".
+        """
+        if _recursive is None:
+            _recursive = util.Set()
+        mapper = _object_mapper(object)
+        key = getattr(object, '_instance_key', None)
+        if key is None:
+            merged = mapper._create_instance(self)
+        else:
+            if key in self.identity_map:
+                merged = self.identity_map[key]
             else:
-                inst = self.get(object.__class__, key[1])
-            if obj is object:
-                instance = inst
-                
-        return instance
+                merged = self.get(mapper.class_, key[1])
+        for prop in mapper.props.values():
+            prop.merge(self, object, merged, _recursive)
+        if key is None:
+            self.save(merged)
+        return merged
                     
     def _save_impl(self, object, **kwargs):
         if hasattr(object, '_instance_key'):
@@ -452,4 +461,4 @@ def object_session(obj):
 
 unitofwork.object_session = object_session
 from sqlalchemy.orm import mapper
-mapper.attribute_manager = attribute_manager
\ No newline at end of file
+mapper.attribute_manager = attribute_manager
