@@ -1,5 +1,5 @@
 # strategies.py
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -73,6 +73,10 @@ class DeferredColumnLoader(LoaderStrategy):
         def lazyload():
             if self._should_log_debug:
                 self.logger.debug("deferred load %s group %s" % (mapperutil.attribute_str(instance, self.key), str(self.group)))
+
+            if not mapper.has_identity(instance):
+                return None
+
             try:
                 pk = self.parent.pks_by_table[self.columns[0].table]
             except KeyError:
@@ -125,17 +129,8 @@ class DeferredOption(StrategizedOption):
 class AbstractRelationLoader(LoaderStrategy):
     def init(self):
         super(AbstractRelationLoader, self).init()
-        self.primaryjoin = self.parent_property.primaryjoin
-        self.secondaryjoin = self.parent_property.secondaryjoin
-        self.secondary = self.parent_property.secondary
-        self.foreignkey = self.parent_property.foreignkey
-        self.mapper = self.parent_property.mapper
-        self.target = self.parent_property.target
-        self.uselist = self.parent_property.uselist
-        self.cascade = self.parent_property.cascade
-        self.attributeext = self.parent_property.attributeext
-        self.order_by = self.parent_property.order_by
-        self.remote_side = self.parent_property.remote_side
+        for attr in ['primaryjoin', 'secondaryjoin', 'secondary', 'foreign_keys', 'mapper', 'select_mapper', 'target', 'select_table', 'loads_polymorphic', 'uselist', 'cascade', 'attributeext', 'order_by', 'remote_side', 'polymorphic_primaryjoin', 'polymorphic_secondaryjoin', 'direction']:
+            setattr(self, attr, getattr(self.parent_property, attr))
         self._should_log_debug = logging.is_debug_enabled(self.logger)
         
     def _init_instance_attribute(self, instance, callable_=None):
@@ -160,7 +155,8 @@ NoLoader.logger = logging.class_logger(NoLoader)
 class LazyLoader(AbstractRelationLoader):
     def init(self):
         super(LazyLoader, self).init()
-        (self.lazywhere, self.lazybinds, self.lazyreverse) = self._create_lazy_clause(self.parent.unjoined_table, self.primaryjoin, self.secondaryjoin, self.foreignkey, self.remote_side)
+        (self.lazywhere, self.lazybinds, self.lazyreverse) = self._create_lazy_clause(self.polymorphic_primaryjoin, self.polymorphic_secondaryjoin, self.remote_side)
+
         # determine if our "lazywhere" clause is the same as the mapper's
         # get() clause.  then we can just use mapper.get()
         self.use_get = not self.uselist and query.Query(self.mapper)._get_clause.compare(self.lazywhere)
@@ -207,7 +203,7 @@ class LazyLoader(AbstractRelationLoader):
             # to possibly save a DB round trip
             if self.use_get:
                 ident = []
-                for primary_key in self.mapper.pks_by_table[self.mapper.mapped_table]:
+                for primary_key in self.select_mapper.pks_by_table[self.select_mapper.mapped_table]:
                     bind = self.lazyreverse[primary_key]
                     ident.append(params[bind.key])
                 return session.query(self.mapper).get(ident)
@@ -244,15 +240,13 @@ class LazyLoader(AbstractRelationLoader):
                 # to load data into it.
                 sessionlib.attribute_manager.reset_instance_attribute(instance, self.key)
 
-    def _create_lazy_clause(self, table, primaryjoin, secondaryjoin, foreignkey, remote_side):
+    def _create_lazy_clause(self, primaryjoin, secondaryjoin, remote_side):
         binds = {}
         reverse = {}
-        def column_in_table(table, column):
-            return table.corresponding_column(column, raiseerr=False, keys_ok=False) is not None
 
-        if remote_side is None or len(remote_side) == 0:
-            remote_side = foreignkey
-            
+        def should_bind(targetcol, othercol):
+            return othercol in remote_side
+
         def find_column_in_expr(expr):
             if not isinstance(expr, sql.ColumnElement):
                 return None
@@ -262,6 +256,13 @@ class LazyLoader(AbstractRelationLoader):
                     columns.append(c)
             expr.accept_visitor(FindColumnInColumnClause())
             return len(columns) and columns[0] or None
+        
+        def col_in_collection(column, collection):
+            for c in collection:
+                if column.shares_lineage(c):
+                    return True
+            else:
+                return False
                 
         def bind_label():
             return "lazy_" + hex(random.randint(0, 65535))[2:]
@@ -270,14 +271,15 @@ class LazyLoader(AbstractRelationLoader):
             rightcol = find_column_in_expr(binary.right)
             if leftcol is None or rightcol is None:
                 return
-            circular = leftcol.table is rightcol.table
-            if ((not circular and column_in_table(table, leftcol)) or (circular and rightcol in remote_side)):
+            if should_bind(leftcol, rightcol):
                 col = leftcol
                 binary.left = binds.setdefault(leftcol,
                         sql.bindparam(bind_label(), None, shortname=leftcol.name, type=binary.right.type))
                 reverse[rightcol] = binds[col]
 
-            if (leftcol is not rightcol) and ((not circular and column_in_table(table, rightcol)) or (circular and leftcol in remote_side)):
+            # the "left is not right" compare is to handle part of a join clause that is "table.c.col1==table.c.col1",
+            # which can happen in rare cases (test/orm/relationships.py RelationTest2)
+            if leftcol is not rightcol and should_bind(rightcol, leftcol):
                 col = rightcol
                 binary.right = binds.setdefault(rightcol,
                         sql.bindparam(bind_label(), None, shortname=rightcol.name, type=binary.left.type))
@@ -286,9 +288,12 @@ class LazyLoader(AbstractRelationLoader):
         lazywhere = primaryjoin.copy_container()
         li = mapperutil.BinaryVisitor(visit_binary)
         lazywhere.accept_visitor(li)
+        
         if secondaryjoin is not None:
+            secondaryjoin = secondaryjoin.copy_container()
             lazywhere = sql.and_(lazywhere, secondaryjoin)
-        LazyLoader.logger.debug("create_lazy_clause " + str(lazywhere))
+ 
+        LazyLoader.logger.info(str(self.parent_property) + " lazy loading clause " + str(lazywhere))
         return (lazywhere, binds, reverse)
 
 LazyLoader.logger = logging.class_logger(LazyLoader)
@@ -337,21 +342,22 @@ class EagerLoader(AbstractRelationLoader):
         """
         def __init__(self, eagerloader, parentclauses=None):
             self.parent = eagerloader
-            self.target = eagerloader.target
-            self.eagertarget = eagerloader.target.alias()
+            self.target = eagerloader.select_table
+            self.eagertarget = eagerloader.select_table.alias()
+            
             if eagerloader.secondary:
                 self.eagersecondary = eagerloader.secondary.alias()
                 self.aliasizer = sql_util.Aliasizer(eagerloader.target, eagerloader.secondary, aliases={
                         eagerloader.target:self.eagertarget,
                         eagerloader.secondary:self.eagersecondary
                         })
-                self.eagersecondaryjoin = eagerloader.secondaryjoin.copy_container()
+                self.eagersecondaryjoin = eagerloader.polymorphic_secondaryjoin.copy_container()
                 self.eagersecondaryjoin.accept_visitor(self.aliasizer)
-                self.eagerprimary = eagerloader.primaryjoin.copy_container()
+                self.eagerprimary = eagerloader.polymorphic_primaryjoin.copy_container()
                 self.eagerprimary.accept_visitor(self.aliasizer)
             else:
-                self.aliasizer = sql_util.Aliasizer(eagerloader.target, aliases={eagerloader.target:self.eagertarget})
-                self.eagerprimary = eagerloader.primaryjoin.copy_container()
+                self.eagerprimary = eagerloader.polymorphic_primaryjoin.copy_container()
+                self.aliasizer = sql_util.Aliasizer(self.target, aliases={self.target:self.eagertarget})
                 self.eagerprimary.accept_visitor(self.aliasizer)
 
             if parentclauses is not None:
@@ -366,15 +372,11 @@ class EagerLoader(AbstractRelationLoader):
 
         def _aliasize_orderby(self, orderby, copy=True):
             if copy:
-                orderby = [o.copy_container() for o in util.to_list(orderby)]
+                return self.aliasizer.copy_and_process(util.to_list(orderby))
             else:
                 orderby = util.to_list(orderby)
-            for i in range(0, len(orderby)):
-                if isinstance(orderby[i], schema.Column):
-                    orderby[i] = self.eagertarget.corresponding_column(orderby[i])
-                else:
-                    orderby[i].accept_visitor(self.aliasizer)
-            return orderby
+                self.aliasizer.process_list(orderby)
+                return orderby
 
         def _create_decorator_row(self):
             class EagerRowAdapter(object):
@@ -443,6 +445,8 @@ class EagerLoader(AbstractRelationLoader):
         except KeyError:
             clauses = EagerLoader.AliasedClauses(self, parentclauses)
             self.clauses[parentclauses] = clauses
+            
+        if context.mapper not in self.clauses_by_lead_mapper:
             self.clauses_by_lead_mapper[context.mapper] = clauses
 
         if self.secondaryjoin is not None:
@@ -460,34 +464,60 @@ class EagerLoader(AbstractRelationLoader):
             clauses._aliasize_orderby(statement.order_by_clause, False)
                 
         statement.append_from(statement._outerjoin)
-        for value in self.mapper.props.values():
-            value.setup(context, eagertable=clauses.eagertarget, parentclauses=clauses, parentmapper=self.mapper)
+        for value in self.select_mapper.props.values():
+            value.setup(context, eagertable=clauses.eagertarget, parentclauses=clauses, parentmapper=self.select_mapper)
+
+    def _create_row_processor(self, selectcontext, row):
+        """create a 'row processing' function that will apply eager aliasing to the row.
+        
+        also check that an identity key can be retrieved from the row, else return None."""
+        # check for a user-defined decorator in the SelectContext (which was set up by the contains_eager() option)
+        if selectcontext.attributes.has_key((EagerLoader, self.parent_property)):
+            # custom row decoration function, placed in the selectcontext by the 
+            # contains_eager() mapper option
+            decorator = selectcontext.attributes[(EagerLoader, self.parent_property)]
+            if decorator is None:
+                decorator = lambda row: row
+        else:
+            try:
+                # decorate the row according to the stored AliasedClauses for this eager load
+                clauses = self.clauses_by_lead_mapper[selectcontext.mapper]
+                decorator = clauses._row_decorator
+            except KeyError, k:
+                # no stored AliasedClauses: eager loading was not set up in the query and
+                # AliasedClauses never got initialized
+                return None
+
+        try:
+            decorated_row = decorator(row)
+            # check for identity key
+            identity_key = self.mapper.identity_key_from_row(decorated_row)
+            # and its good
+            return decorator
+        except KeyError, k:
+            # no identity key - dont return a row processor, will cause a degrade to lazy
+            if self._should_log_debug:
+                self.logger.debug("could not locate identity key from row '%s'; missing column '%s'" % (repr(decorated_row), str(k)))
+            return None
 
     def process_row(self, selectcontext, instance, row, identitykey, isnew):
         """receive a row.  tell our mapper to look for a new object instance in the row, and attach
         it to a list on the parent instance."""
-
         if self in selectcontext.recursion_stack:
             return
         
         try:
-            # decorate the row according to the stored AliasedClauses for this eager load,
-            # or look for a user-defined decorator in the SelectContext (which was set up by the contains_eager() option)
-            if selectcontext.attributes.has_key((EagerLoader, self.parent_property)):
-                # custom row decoration function, placed in the selectcontext by the 
-                # contains_eager() mapper option
-                decorator = selectcontext.attributes[(EagerLoader, self.parent_property)]
-                if decorator is None:
-                    decorated_row = row
-                else:
-                    decorated_row = decorator(row)
-            else:
-                clauses = self.clauses_by_lead_mapper[selectcontext.mapper]
-                decorated_row = clauses._decorate_row(row)
-            # check for identity key
-            identity_key = self.mapper.identity_key_from_row(decorated_row)
+            # check for row processor
+            row_processor = selectcontext.attributes[id(self)]
         except KeyError:
-            # else degrade to a lazy loader
+            # create a row processor function and cache it in the context
+            row_processor = self._create_row_processor(selectcontext, row)
+            selectcontext.attributes[id(self)] = row_processor
+            
+        if row_processor is not None:
+            decorated_row = row_processor(row)
+        else:
+            # row_processor was None: degrade to a lazy loader
             if self._should_log_debug:
                 self.logger.debug("degrade to lazy loader on %s" % mapperutil.attribute_str(instance, self.key))
             self.parent_property._get_strategy(LazyLoader).process_row(selectcontext, instance, row, identitykey, isnew)
@@ -550,10 +580,20 @@ class EagerLazyOption(StrategizedOption):
 EagerLazyOption.logger = logging.class_logger(EagerLazyOption)
 
 class RowDecorateOption(PropertyOption):
-    def __init__(self, key, decorator=None):
+    def __init__(self, key, decorator=None, alias=None):
         super(RowDecorateOption, self).__init__(key)
         self.decorator = decorator
+        self.alias = alias
     def process_selection_property(self, context, property):
+        if self.alias is not None and self.decorator is None:
+            if isinstance(self.alias, basestring):
+                self.alias = property.target.alias(self.alias)
+            def decorate(row):
+                d = {}
+                for c in property.target.columns:
+                    d[c] = row[self.alias.corresponding_column(c)]
+                return d
+            self.decorator = decorate
         context.attributes[(EagerLoader, property)] = self.decorator
 RowDecorateOption.logger = logging.class_logger(RowDecorateOption)
         
