@@ -1,5 +1,5 @@
 # pool.py - Connection pooling for SQLAlchemy
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -16,7 +16,7 @@ try:
 except:
     import pickle
     
-from sqlalchemy import util, exceptions, logging
+from sqlalchemy import exceptions, logging
 from sqlalchemy import queue as Queue
 
 try:
