@@ -1,4 +1,4 @@
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -10,7 +10,7 @@ from sqlalchemy import types as sqltypes
 import string, re, random, sets
 
 
-__all__ = ['text', 'table', 'column', 'func', 'select', 'update', 'insert', 'delete', 'join', 'and_', 'or_', 'not_', 'between_', 'case', 'cast', 'union', 'union_all', 'except_', 'except_all', 'intersect', 'intersect_all', 'null', 'desc', 'asc', 'outerjoin', 'alias', 'subquery', 'literal', 'bindparam', 'exists', 'extract','AbstractDialect', 'ClauseParameters', 'ClauseVisitor', 'Executor', 'Compiled', 'ClauseElement', 'ColumnElement', 'ColumnCollection', 'FromClause', 'TableClause', 'Select', 'Alias', 'CompoundSelect','Join', 'Selectable']
+__all__ = ['text', 'table', 'column', 'literal_column', 'func', 'select', 'update', 'insert', 'delete', 'join', 'and_', 'or_', 'not_', 'between_', 'case', 'cast', 'union', 'union_all', 'except_', 'except_all', 'intersect', 'intersect_all', 'null', 'desc', 'asc', 'outerjoin', 'alias', 'subquery', 'literal', 'bindparam', 'exists', 'extract','AbstractDialect', 'ClauseParameters', 'ClauseVisitor', 'Executor', 'Compiled', 'ClauseElement', 'ColumnElement', 'ColumnCollection', 'FromClause', 'TableClause', 'Select', 'Alias', 'CompoundSelect','Join', 'Selectable']
 
 def desc(column):
     """return a descending ORDER BY clause element, e.g.:
@@ -219,11 +219,15 @@ def label(name, obj):
     """returns a _Label object for the given selectable, used in the column list for a select statement."""
     return _Label(name, obj)
     
-def column(text, table=None, type=None):
-    """returns a textual column clause, relative to a table.  this is also the primitive version of
+def column(text, table=None, type=None, **kwargs):
+    """return a textual column clause, relative to a table.  this is also the primitive version of
     a schema.Column which is a subclass. """
-    return _ColumnClause(text, table, type)
+    return _ColumnClause(text, table, type, **kwargs)
 
+def literal_column(text, table=None, type=None, **kwargs):
+    """return a textual column clause with the 'literal' flag set.  this column will not be quoted"""
+    return _ColumnClause(text, table, type, is_literal=True, **kwargs)
+    
 def table(name, *columns):
     """returns a table clause.  this is a primitive version of the schema.Table object, which is a subclass
     of this object."""
@@ -274,6 +278,8 @@ class _FunctionGateway(object):
     """returns a callable based on an attribute name, which then returns a _Function 
     object with that name."""
     def __getattr__(self, name):
+        if name[-1] == '_':
+            name = name[0:-1]
         return getattr(_FunctionGenerator(), name)
 func = _FunctionGateway()
 
@@ -385,7 +391,8 @@ class Compiled(ClauseVisitor):
         self.statement = statement
         self.parameters = parameters
         self.engine = engine
-    
+        self.can_execute = statement.supports_execution()
+        
     def compile(self):
         self.statement.accept_visitor(self)
         self.after_compile()
@@ -437,6 +444,10 @@ class ClauseElement(object):
         """accept a ClauseVisitor and call the appropriate visit_xxx method."""
         raise NotImplementedError(repr(self))
 
+    def supports_execution(self):
+        """return True if this clause element represents a complete executable statement"""
+        return False
+        
     def copy_container(self):
         """return a copy of this ClauseElement, iff this ClauseElement contains other ClauseElements.  
         
@@ -562,7 +573,7 @@ class _CompareMixin(object):
     def between(self, cleft, cright):
         return _BooleanExpression(self, and_(self._check_literal(cleft), self._check_literal(cright)), 'BETWEEN')
     def op(self, operator):
-        return lambda other: self._compare(operator, other)
+        return lambda other: self._operate(operator, other)
     # and here come the math operators:
     def __add__(self, other):
         return self._operate('+', other)
@@ -631,7 +642,7 @@ class ColumnElement(Selectable, _CompareMixin):
     may correspond to several TableClause-attached columns)."""
     
     primary_key = property(lambda self:getattr(self, '_primary_key', False), doc="primary key flag.  indicates if this Column represents part or whole of a primary key.")
-    foreign_keys = property(lambda self:getattr(self, '_foreign_keys', []), doc="foreign key accessor.  points to a ForeignKey object which represents a Foreign Key placed on this column's ultimate ancestor.")
+    foreign_keys = property(lambda self:getattr(self, '_foreign_keys', []), doc="foreign key accessor.  points to a list of ForeignKey objects which represents a Foreign Key placed on this column's ultimate ancestor.")
     columns = property(lambda self:[self], doc="Columns accessor which just returns self, to provide compatibility with Selectable objects.")
     def _one_fkey(self):
         if len(self._foreign_keys):
@@ -676,6 +687,9 @@ class ColumnCollection(util.OrderedProperties):
     
     overrides the __eq__() method to produce SQL clauses between sets of
     correlated columns."""
+    def __init__(self, *cols):
+        super(ColumnCollection, self).__init__()
+        [self.add(c) for c in cols]
     def add(self, column):
         """add a column to this collection.
         
@@ -689,7 +703,12 @@ class ColumnCollection(util.OrderedProperties):
                 if c.shares_lineage(local):
                     l.append(c==local)
         return and_(*l)
-             
+    def contains_column(self, col):
+        # have to use a Set here, because it will compare the identity 
+        # of the column, not just using "==" for comparison which will always return a
+        # "True" value (i.e. a BinaryClause...)
+        return col in util.Set(self)
+        
 class FromClause(Selectable):
     """represents an element that can be used within the FROM clause of a SELECT statement."""
     def __init__(self, name=None):
@@ -723,9 +742,17 @@ class FromClause(Selectable):
         if not hasattr(self, '_oid_column'):
             self._oid_column = self._locate_oid_column()
         return self._oid_column
-    def corresponding_column(self, column, raiseerr=True, keys_ok=False):
+    def corresponding_column(self, column, raiseerr=True, keys_ok=False, require_exact=False):
         """given a ColumnElement, return the ColumnElement object from this 
         Selectable which corresponds to that original Column via a proxy relationship."""
+        if require_exact:
+            if self.columns.get(column.name) is column:
+                return column
+            else:
+                if not raiseerr:
+                    return None
+                else:
+                    raise exceptions.InvalidRequestError("Column instance '%s' is not directly present in table '%s'" % (str(column), str(column.table)))
         for c in column.orig_set:
             try:
                 return self.original_columns[c]
@@ -734,7 +761,7 @@ class FromClause(Selectable):
         else:
             if keys_ok:
                 try:
-                    return self.c[column.key]
+                    return self.c[column.name]
                 except KeyError:
                     pass
             if not raiseerr:
@@ -870,7 +897,9 @@ class _TextClause(ClauseElement):
         visitor.visit_textclause(self)
     def _get_from_objects(self):
         return []
-
+    def supports_execution(self):
+        return True
+        
 class _Null(ColumnElement):
     """represents the NULL keyword in a SQL statement. public contstructor is the
     null() function."""
@@ -1073,8 +1102,14 @@ class _BinaryClause(ClauseElement):
             self.left.compare(other.left) and self.right.compare(other.right)
         )
 
-class _BooleanExpression(_BinaryClause):
-    """represents a boolean expression, which is only useable in WHERE criterion."""
+class _BinaryExpression(_BinaryClause, ColumnElement):
+    """represents a binary expression, which can be in a WHERE criterion or in the column list 
+    of a SELECT.  By adding "ColumnElement" to its inherited list, it becomes a Selectable
+    unit which can be placed in the column list of a SELECT."""
+    pass
+
+class _BooleanExpression(_BinaryExpression):
+    """represents a boolean expression."""
     def __init__(self, *args, **kwargs):
         self.negate = kwargs.pop('negate', None)
         super(_BooleanExpression, self).__init__(*args, **kwargs)
@@ -1084,13 +1119,6 @@ class _BooleanExpression(_BinaryClause):
         else:
             return super(_BooleanExpression, self)._negate()
         
-class _BinaryExpression(_BinaryClause, ColumnElement):
-    """represents a binary expression, which can be in a WHERE criterion or in the column list 
-    of a SELECT.  By adding "ColumnElement" to its inherited list, it becomes a Selectable
-    unit which can be placed in the column list of a SELECT."""
-    pass
-    
-        
 class Join(FromClause):
     def __init__(self, left, right, onclause=None, isouter = False):
         self.left = left._selectable()
@@ -1175,8 +1203,9 @@ class Alias(FromClause):
                 alias = alias[0:15]
             alias = alias + "_" + hex(random.randint(0, 65535))[2:]
         self.name = alias
-        self.case_sensitive = getattr(baseselectable, "case_sensitive", alias.lower() != alias)
-        
+        self.case_sensitive = getattr(baseselectable, "case_sensitive", True)
+    def supports_execution(self):
+        return self.original.supports_execution()    
     def _locate_oid_column(self):
         if self.selectable.oid_column is not None:
             return self.selectable.oid_column._make_proxy(self)
@@ -1208,7 +1237,7 @@ class _Label(ColumnElement):
         while isinstance(obj, _Label):
             obj = obj.obj
         self.obj = obj
-        self.case_sensitive = getattr(obj, "case_sensitive", name.lower() != name)
+        self.case_sensitive = getattr(obj, "case_sensitive", True)
         self.type = sqltypes.to_instance(type)
         obj.parens=True
     key = property(lambda s: s.name)
@@ -1226,12 +1255,14 @@ legal_characters = util.Set(string.ascii_letters + string.digits + '_')
 class _ColumnClause(ColumnElement):
     """represents a textual column clause in a SQL statement.  May or may not
     be bound to an underlying Selectable."""
-    def __init__(self, text, selectable=None, type=None, _is_oid=False):
+    def __init__(self, text, selectable=None, type=None, _is_oid=False, case_sensitive=True, is_literal=False):
         self.key = self.name = text
         self.table = selectable
         self.type = sqltypes.to_instance(type)
         self._is_oid = _is_oid
         self.__label = None
+        self.case_sensitive = case_sensitive
+        self.is_literal = is_literal
     def _get_label(self):
         if self.__label is None:
             if self.table is not None and self.table.named_with_column():
@@ -1329,6 +1360,8 @@ class TableClause(FromClause):
 
 class _SelectBaseMixin(object):
     """base class for Select and CompoundSelects"""
+    def supports_execution(self):
+        return True
     def order_by(self, *clauses):
         if len(clauses) == 1 and clauses[0] is None:
             self.order_by_clause = ClauseList()
@@ -1460,14 +1493,17 @@ class Select(_SelectBaseMixin, FromClause):
         if columns is not None:
             for c in columns:
                 self.append_column(c)
-            
+
+        for f in from_obj:
+            self.append_from(f)
+
+        # whereclauses must be appended after the columns/FROM, since it affects
+        # the correlation of subqueries.  see test/sql/select.py SelectTest.testwheresubquery
         if whereclause is not None:
             self.append_whereclause(whereclause)
         if having is not None:
             self.append_having(having)
             
-        for f in from_obj:
-            self.append_from(f)
     
     class _CorrelatedVisitor(ClauseVisitor):
         """visits a clause, locates any Select clauses, and tells them that they should
@@ -1493,7 +1529,7 @@ class Select(_SelectBaseMixin, FromClause):
                 
     def append_column(self, column):
         if _is_literal(column):
-            column = _ColumnClause(str(column), self)
+            column = literal_column(str(column), table=self)
 
         self._raw_columns.append(column)
 
@@ -1574,7 +1610,13 @@ class Select(_SelectBaseMixin, FromClause):
         else:
             return None
 
-    froms = property(lambda self: self.__froms.difference(self.__hide_froms).difference(self.__correlated), doc="""a collection containing all elements of the FROM clause""")
+    def _calc_froms(self):
+        f = self.__froms.difference(self.__hide_froms)
+        if (len(f) > 1):
+            return f.difference(self.__correlated)
+        else:
+            return f
+    froms = property(_calc_froms, doc="""a collection containing all elements of the FROM clause""")
 
     def accept_visitor(self, visitor):
         for f in self.froms:
@@ -1618,6 +1660,8 @@ class Select(_SelectBaseMixin, FromClause):
 
 class _UpdateBase(ClauseElement):
     """forms the base for INSERT, UPDATE, and DELETE statements."""
+    def supports_execution(self):
+        return True
     def _process_colparams(self, parameters):
         """receives the "values" of an INSERT or UPDATE statement and constructs
         appropriate bind parameters."""
