@@ -49,6 +49,7 @@ class ReflectionTest(PersistTest):
             Column('test_passivedefault2', Integer, PassiveDefault("5")),
             Column('test_passivedefault3', deftype2, PassiveDefault(defval2)),
             Column('test9', Binary(100)),
+            Column('test_numeric', Numeric(None, None)),
             mysql_engine='InnoDB'
         )
 
@@ -58,7 +59,8 @@ class ReflectionTest(PersistTest):
             Column('email_address', String(20)),
             mysql_engine='InnoDB'
         )
-
+        meta.drop_all()
+        
         users.create()
         addresses.create()
 
@@ -169,7 +171,6 @@ class ReflectionTest(PersistTest):
                 autoload=True)
             u2 = Table('users', meta2, autoload=True)
 
-            print "ITS", list(a2.primary_key)
             assert list(a2.primary_key) == [a2.c.id]
             assert list(u2.primary_key) == [u2.c.id]
             assert u2.join(a2).onclause == u2.c.id==a2.c.id
@@ -304,6 +305,7 @@ class ReflectionTest(PersistTest):
             'multi', meta, 
             Column('multi_id', Integer, primary_key=True),
             Column('multi_rev', Integer, primary_key=True),
+            Column('multi_hoho', Integer, primary_key=True),
             Column('name', String(50), nullable=False),
             Column('val', String(100)),
             mysql_engine='InnoDB'
@@ -312,10 +314,12 @@ class ReflectionTest(PersistTest):
             Column('id', Integer, primary_key=True),
             Column('foo', Integer),
             Column('bar', Integer),
+            Column('lala', Integer),
             Column('data', String(50)),
-            ForeignKeyConstraint(['foo', 'bar'], ['multi.multi_id', 'multi.multi_rev']),
+            ForeignKeyConstraint(['foo', 'bar', 'lala'], ['multi.multi_id', 'multi.multi_rev', 'multi.multi_hoho']),
             mysql_engine='InnoDB'
         )
+        assert table.c.multi_hoho
         meta.create_all()
         meta.clear()
         
@@ -327,11 +331,10 @@ class ReflectionTest(PersistTest):
             print table2
             j = join(table, table2)
             print str(j.onclause)
-            self.assert_(and_(table.c.multi_id==table2.c.foo, table.c.multi_rev==table2.c.bar).compare(j.onclause))
+            self.assert_(and_(table.c.multi_id==table2.c.foo, table.c.multi_rev==table2.c.bar, table.c.multi_hoho==table2.c.lala).compare(j.onclause))
 
         finally:
-            pass
-#            meta.drop_all()
+            meta.drop_all()
 
     def testcheckfirst(self):
         meta = BoundMetaData(testbase.db)
@@ -352,14 +355,15 @@ class ReflectionTest(PersistTest):
         finally:
             meta.drop_all()
             
-    def testtoengine(self):
+    def testtometadata(self):
         meta = MetaData('md1')
         meta2 = MetaData('md2')
         
         table = Table('mytable', meta,
             Column('myid', Integer, primary_key=True),
             Column('name', String, nullable=False),
-            Column('description', String(30)),
+            Column('description', String(30), CheckConstraint("description='hi'")),
+            UniqueConstraint('name')
         )
         
         table2 = Table('othertable', meta,
@@ -379,6 +383,20 @@ class ReflectionTest(PersistTest):
         assert [x.name for x in table.primary_key] == [x.name for x in table_c.primary_key]
         assert list(table2_c.c.myid.foreign_keys)[0].column is table_c.c.myid
         assert list(table2_c.c.myid.foreign_keys)[0].column is not table.c.myid
+        for c in table_c.c.description.constraints:
+            if isinstance(c, CheckConstraint):
+                break
+        else:
+            assert False
+        assert c.sqltext=="description='hi'"
+        
+        for c in table_c.constraints:
+            if isinstance(c, UniqueConstraint):
+                break
+        else:
+            assert False
+        assert c.columns.contains_column(table_c.c.name)
+        assert not c.columns.contains_column(table.c.name)
         
     # mysql throws its own exception for no such table, resulting in 
     # a sqlalchemy.SQLError instead of sqlalchemy.NoSuchTableError.
@@ -488,9 +506,37 @@ class SchemaTest(PersistTest):
         print buf
         assert buf.index("CREATE TABLE someschema.table1") > -1
         assert buf.index("CREATE TABLE someschema.table2") > -1
-         
-        
+    
+    @testbase.supported('postgres')
+    def testpg(self):
+        """note: this test requires that the 'test_schema' schema be separate and accessible by the test user"""
         
+        meta1 = BoundMetaData(testbase.db)
+        users = Table('users', meta1,
+            Column('user_id', Integer, primary_key = True),
+            Column('user_name', String(30), nullable = False),
+            schema="test_schema"
+            )
+
+        addresses = Table('email_addresses', meta1,
+            Column('address_id', Integer, primary_key = True),
+            Column('remote_user_id', Integer, ForeignKey(users.c.user_id)),
+            Column('email_address', String(20)),
+            schema="test_schema"
+        )
+        meta1.create_all()
+        try:
+            meta2 = BoundMetaData(testbase.db)
+            addresses = Table('email_addresses', meta2, autoload=True, schema="test_schema")
+            users = Table('users', meta2, mustexist=True, schema="test_schema")
+
+            print users
+            print addresses
+            j = join(users, addresses)
+            print str(j.onclause)
+            self.assert_((users.c.user_id==addresses.c.remote_user_id).compare(j.onclause))
+        finally:
+            meta1.drop_all()
         
 if __name__ == "__main__":
     testbase.main()        
