@@ -16,6 +16,8 @@ def suite():
         'orm.cascade',
         'orm.relationships',
         'orm.association',
+        'orm.merge',
+        'orm.memusage',
         
         'orm.cycles',
         'orm.poly_linked_list',
@@ -27,6 +29,9 @@ def suite():
         'orm.inheritance',
         'orm.inheritance2',
         'orm.inheritance3',
+        'orm.inheritance4',
+        'orm.inheritance5',
+        'orm.single',
         'orm.polymorph'        
         )
     alltests = unittest.TestSuite()
