@@ -7,16 +7,21 @@ import sqlalchemy.databases.sqlite as sqllite
 import tables
 from sqlalchemy import *
 from sqlalchemy.engine import ResultProxy, RowProxy
+from sqlalchemy import exceptions
 
 class QueryTest(PersistTest):
     
     def setUpAll(self):
-        global users, metadata
+        global users, addresses, metadata
         metadata = BoundMetaData(testbase.db)
         users = Table('query_users', metadata,
             Column('user_id', INT, primary_key = True),
             Column('user_name', VARCHAR(20)),
         )
+        addresses = Table('query_addresses', metadata, 
+            Column('address_id', Integer, primary_key=True),
+            Column('user_id', Integer, ForeignKey('query_users.user_id')),
+            Column('address', String(30)))
         metadata.create_all()
     
     def setUp(self):
@@ -154,6 +159,16 @@ class QueryTest(PersistTest):
         self.users.insert().execute(user_id=7, user_name='fido')
         r = self.users.select(limit=3, order_by=[self.users.c.user_id]).execute().fetchall()
         self.assert_(r == [(1, 'john'), (2, 'jack'), (3, 'ed')], repr(r))
+        
+    @testbase.unsupported('mssql')
+    def testselectlimitoffset(self):
+        self.users.insert().execute(user_id=1, user_name='john')
+        self.users.insert().execute(user_id=2, user_name='jack')
+        self.users.insert().execute(user_id=3, user_name='ed')
+        self.users.insert().execute(user_id=4, user_name='wendy')
+        self.users.insert().execute(user_id=5, user_name='laura')
+        self.users.insert().execute(user_id=6, user_name='ralph')
+        self.users.insert().execute(user_id=7, user_name='fido')
         r = self.users.select(limit=3, offset=2, order_by=[self.users.c.user_id]).execute().fetchall()
         self.assert_(r==[(3, 'ed'), (4, 'wendy'), (5, 'laura')])
         r = self.users.select(offset=5, order_by=[self.users.c.user_id]).execute().fetchall()
@@ -205,6 +220,12 @@ class QueryTest(PersistTest):
         self.assertEqual(len(r), 1)
         r.close()
     
+    def test_cant_execute_join(self):
+        try:
+            users.join(addresses).execute()
+        except exceptions.ArgumentError, e:
+            assert str(e) == """Not an executeable clause: query_users JOIN query_addresses ON query_users.user_id = query_addresses.user_id"""
+            
     def test_functions(self):
         x = testbase.db.func.current_date().execute().scalar()
         y = testbase.db.func.current_date().select().execute().scalar()
@@ -292,7 +313,7 @@ class QueryTest(PersistTest):
         self.assertEqual(r[1], 1)
         self.assertEqual(r.keys(), ['user_name', 'user_id'])
         self.assertEqual(r.values(), ['foo', 1])
-       
+    
     @testbase.unsupported('oracle', 'firebird') 
     def test_column_accessor_shadow(self):
         meta = BoundMetaData(testbase.db)
